
/*
** OIDS.H
**
** Header file for Xoids.
**
** Tim Ebling
** tebling@oce.orst.edu
**
*/

/*
 * standard colour indices
 */
#define	BLACK		0
#define	RED			1
#define	GREEN		2
#define	YELLOW		3
#define	BLUE		4
#define	MAGENTA		5
#define	CYAN		6
#define	WHITE		7

// Be specific

typedef BBitmap* 	Pixmap;

#define XRectangle	BRect
#define	XPoint		BPoint

/*
 * How to convert degrees to radians
 */
#ifndef PI
#define	PI	3.14159265358979
#endif

#define D2R	(PI / 180.0)


#define WINDOW_HEIGHT	480
#define WINDOW_WIDTH	640

#define MAPS_PER_360	32
#define OIDS_PER_360	32
#define OID_DIVIDE		2
#define DELTA_ANG		(2 * PI / MAPS_PER_360)

#define MAX_PLAYERS		2
#define P_MAX_SPEED		8
#define N_THRUST_PTS 	3
#define HYPER_DELAY		20

#define SLURB_SPEED		4
#define H_MAX_SPEED		12
#define INIT_BIG_OIDS	3
#define MAX_BIG_OIDS	32

#define BIG_OID_ROT		10
#define BO_MAX_SPEED	4

#define MED_OID_ROT		5
#define MO_MAX_SPEED	8

#define SML_OID_ROT		2
#define SO_MAX_SPEED	16

#define NUM_SHARDS		5
#define CARNAGE_SPEED	1.5
#define CARNAGE_LIFE	100
#define DYING_TIME		15
#define DEATH_THRESH	5

#define TIMER_VAL		30
#define NUM_STARS		40
#define LINK_LENGTH		40

#define NORMAL			0
#define DYING			1
#define WARPING			2
#define REVIVING		3
#define EXPLODING		4
#define DEAD			5

#define GUN				0
#define LASER			1

#define KEY_THRUST1		0x0001
#define KEY_LEFT1		0x0002
#define KEY_RIGHT1		0x0004
#define KEY_FIRE1		0x0008
#define KEY_HYPER1		0x0010
#define KEY_THRUST2		0x0020
#define KEY_LEFT2		0x0040
#define KEY_RIGHT2		0x0080
#define KEY_FIRE2		0x0100
#define KEY_HYPER2		0x0200
#define KEY_QUIT		0x0400
#define KEY_PAUSE		0x0800
#define KEY_ESC			0x1000

#define UPARROW 		B_UP_ARROW
#define DOWNARROW 		B_DOWN_ARROW
#define LEFTARROW 		B_LEFT_ARROW
#define RIGHTARROW 		B_RIGHT_ARROW
#define ENTER 			B_ENTER

/*
 * max number of characters in a font name
 */

typedef struct vdev 
{
	char	initialised,
			clipoff,
			inobject,
			inpolygon,
			upset,			/* is up vector set */
			cpVvalid,		/* is the current device position valid */
			inbackbuffer,		/* are we in the backbuffer */
			clipplanes;		/* active clipping planes */
	void	(*pmove)(),		/* Polygon moves */
			(*pdraw)();		/* Polygon draws */
	float	hheight, hwidth;	/* hardware character height, width */
	int		depth,			/* # bit planes on screen */
			maxVx, minVx,
			maxVy, minVy,
			sizeX, sizeY, 		/* size of square on screen */
			sizeSx, sizeSy,		/* side in x, side in y (# pixels) */
			cpVx, cpVy;
} Device;

extern Device	vdevice;		/* device structure */

struct Shot 
{
	unsigned int	clock;

	int				x,
					y,
					ox,
					oy;

	int				pts_x[5],
					pts_y[5],
					num_kill_pts,
					kill_pts[2];

	float			x_vec,
					y_vec;

};

struct Wpn 
{
	int				pts_x[5],
					pts_y[5];

	unsigned int	max_shots,
					shot_speed,
					shot_life,
					inhibit_time,
					size,
					num_kill_pts,
					kill_pts[2];

};
	

typedef struct spr 
{
	unsigned int	width,
					height,
					wo2,
					ho2,
					num_members,
					num_pixmaps,
					shots,
					shot_color,
					burst_color,
					value,
					score,
					shots_fired,
					engine,
					hits,
					num_death_sprites,
					rotation,
					thrust,
					state,
					state_ctr,
					max_dying,
					max_exploding,
					bounced;

	int				rot_dir,
					curr_map,
					lives;

	float			ox,
					oy,
					x,
					y;
				
	float			x_vec,
					y_vec,
					max_speed,
					point_angle,
					delta_angle,
					mass;

	char			name[40];

	unsigned int	wpn;

	struct Shot		**S;

	char			***pixmap_data;

	Pixmap			*pixmaps,
					*clipmasks,
					*orig_pixmaps,
					*orig_clipmasks;

	struct spr		*death_sprite;
	struct spr		*next;
	struct spr		*next_draw;

	void			(*kill_func)(void *);

} Sprite;


struct explosion {

	unsigned int		radius,
				max_radius,
				color;

	int			x,
				y,
				dir,
				x_vec,
				y_vec;

};

struct linkage {

	float			x,
				y,
				x_vec,
				y_vec,
				ang,
				ang_vec;

	unsigned int		length;

};


/* Be specific */

// Types
typedef unsigned char 	UBYTE;
typedef unsigned short 	UWORD;
typedef unsigned long 	ULONG;
typedef char 			BYTE;
typedef short 			WORD;
typedef long 			LONG;

class 					OidsWindow;
class					OidsView;

// Application object
class Oids : public BApplication 
{
	public:
						Oids();
			void 		ReadyToRun(void);
			bool 		QuitRequested(void);
			void 		AboutRequested(void);
	static	void 		init_game(void);

			int 		i;
			Sprite 		*the_obj;
			OidsWindow	*the_window;
};

class OidsView : public BView
{
	public:
						OidsView(BRect frame);
		
			void		Draw(BRect update);
			void 		KeyDown(ulong aKey);
			void		RefreshAll(void);

};

// Window object
class OidsWindow : public BWindow 
{
	public:
	
						OidsWindow(short width, short height);
			void 		MessageReceived(BMessage *message);
			bool 		QuitRequested(void);
	static	void		Refresh(OidsWindow *w);

	private:

			OidsView	*main_view;	
			ULONG		xmod;

			int 		i;
			Sprite 		*the_obj;
};

extern BBitmap			*bmp;
extern BView			*view;
extern OidsView			*window_view;
extern uchar			colors[256],*bits;		
extern ulong			globalkey;

extern Pixmap			blist[MAX_BIG_OIDS * (OID_DIVIDE+2) * MAPS_PER_360];
extern ulong			bcount;

/* External variables */

extern Sprite	*P;
extern Sprite  *Slrb;
extern Sprite	*Homer;
extern Sprite	*Resur;
extern Sprite  *Big_O;
extern Sprite 	*Med_O[MAX_BIG_OIDS];
extern Sprite 	*Sml_O[MAX_BIG_OIDS * OID_DIVIDE];
extern Sprite  *Shrd[4];
extern Sprite  *Flame;
extern Sprite	*Power_Up;
extern Sprite	*Meta;

extern int	keyboard_state,
	goal_score;

extern struct explosion burst[250];

extern struct linkage link_;

extern struct Wpn Weapon[];

extern int thrust_pts_x[];
extern int thrust_pts_y[];

extern int hot_spot_x;
extern int hot_spot_y;

extern char *mm_items[];

extern char *om_items[];

extern char *goal_str[];

extern int goal_array[];

extern int mm_num_items;
extern int om_num_items;

extern float cos_table[MAPS_PER_360];
extern float sin_table[MAPS_PER_360];

extern int r_num;	/* Global random number */

extern int delay;	/* Main loop delay factor */

extern unsigned int 	timer,		/* Game countdown clock */
		shot_clock;	/* Countdown from shot fired */

extern XRectangle clip_rect;	/* Clipping rectangle for scoreboard */

extern XPoint		starfield[NUM_STARS];	/* Coordinates of stars */

extern unsigned int	num_bursts,	/* Number of bursts */
		num_players,	/* Number of players */
		oids_shot,	/* Number of oids currently destroyed */
		level,		/* Play level */
		warp_levels,	/* Warping to a new level? */
		extra_man_scr,	/* Score to subtract for "bonus ship"? */
		coop,		/* Flag for co-op play */
		game_over,	/* Flag for end of game */
		leave;		/* Flag to leave game loop */

 
/*
** Function prototypes
*/

void X_init_pixmaps();
char **X_rotate_pixmap_data(char **orig_pmap, unsigned int h,unsigned int w,unsigned int alpha);
char *X_rotate_bitmap();
void X_update(unsigned int game_flag);
void X_draw_one_object(Sprite *obj,int x, int y);
int X_checkkey();
int X_get_menu_keys();
int X_getkey();
void X_backbuf();
void X_frontbuf();
void X_clear();
void X_color(int ind);
void X_string(char *s, int y);
void X_exit();
void X_spin_pixmap360(Sprite *obj);
void X_init();
void X_draw_object(Sprite *obj);
void X_draw_thrust(Sprite *obj);
void X_draw_shots(Sprite *obj);
void X_draw_bursts(int flag);
void X_copy_object_to_window(Sprite *obj);
void X_clear_object(Sprite *obj);
void X_clear_shots(Sprite *obj);
void X_draw_bursts();
void X_check_keypress();
void X_clear_one_object(Sprite *obj);
void X_flash_screen(int d);
void X_draw_status_bar();
int X_mapcolor(int i, int r, int g, int b);
void X_swapbuf();
void X_update_status_bar();
void X_copy_status_bar_to_window();
void X_draw_stars(int col, int b);
void X_draw_link(Sprite *obj,int col);
void X_clear_key_buffer();
void X_point(int x, int y);

void init_all_objects(int pass);
void free_all_objects();
Sprite *init_object(int elems,int n_maps, char **data);
struct Shot **init_shots();
void create_new_shot(Sprite *obj);
Sprite *update_shots(Sprite *obj);
void erase_shot(Sprite *obj, int n);
void update_object(Sprite *obj);
float random_num(float low, float high);
void rotate_oids();
Sprite *check_collision(Sprite *obj1, Sprite *ob2, int bnce, int kill);
void explode_object(Sprite *obj);
void revive_player(Sprite *obj);
int bounce_objects(Sprite *obj1, Sprite *obj2);
void check_input();
void warp_object(Sprite *obj);
Sprite *shot_collide(struct Shot *this_shot, Sprite *obj);
int simple_collision(Sprite *obj1, Sprite *obj2, int bnce, int kill);
float speed(Sprite *obj);
void update_score();
void kill_shard(Sprite *the_shard);
void do_nothing_for_a_while();
void revive_oid(Sprite *obj, int flag);
void destroy_oid(Sprite *the_oid);
void update_bursts();
void make_burst(Sprite *obj, int dir);
void main_menu();
void draw_game_over();
void pause_game();
void test_for_aliens();
float distance(Sprite *obj1, Sprite *obj2);
void init_starfield();
void draw_level_warp();
void link_players();
void options_menu();
void put_inside_window(Sprite *obj);
void kill_power_up(Sprite *the_pu, Sprite *the_lucky);
void destroy_object(Sprite *obj);
void destroy_homer(Sprite *obj);
void alien_lock_on(Sprite *alien);
void draw_homer_trail(int reset);
