/*
** X11 routines for XOIDS
**
** by Tim Ebling
** tebling@oce.orst.edu
** 
** This is actually a hacked version of the X11 device driver
** from the VOGLE graphics library.  Not much is left of the
** original though :)
**
*/
#include <stdio.h>
#include <math.h>
#include <string.h>

#include "oids.h"

#define MIN(x,y)	((x) < (y) ? (x) : (y))
#define	CMAPSIZE	256

static	int		screen;
static	unsigned long	carray[CMAPSIZE];
static	int		back_used;	/* Have we backbuffered ? */

static	unsigned long	colour;
static	unsigned int	h, w;

static  XRectangle	eng_outlinel,
					eng_outliner;

static unsigned int	wo7, wo40;

Pixmap			blist[MAX_BIG_OIDS * (OID_DIVIDE+2) * MAPS_PER_360];
ulong			bcount=0;

/*
 * X_init
 *
 *	initialises X11 display.
 */
void X_init()
{
	vdevice.sizeX = vdevice.sizeY = MIN(WINDOW_HEIGHT, WINDOW_WIDTH) - 1;
	vdevice.sizeSx = WINDOW_WIDTH;
	vdevice.sizeSy = WINDOW_HEIGHT;

	clip_rect.top = 0;
	clip_rect.left = P->height + 4;
	clip_rect.right = clip_rect.left + vdevice.sizeSx;
	clip_rect.bottom = clip_rect.top + vdevice.sizeSy - (int) (P->height + 4);

	wo7 = (int) (vdevice.sizeSx / 7);
	wo40 = (int) (vdevice.sizeSx / 40);

	eng_outlinel.top = P->width + 150;
	eng_outlinel.left = 7;
	eng_outlinel.bottom = eng_outlinel.top + (int) P->height - 11;
	eng_outlinel.right = eng_outlinel.left + wo7;

	eng_outliner.top = 6 * wo7 - 5;
	eng_outliner.left = 7;
	eng_outliner.bottom = eng_outliner.top + P->height - 11;
	eng_outliner.right = eng_outliner.left + wo7;

	back_used = 0;
}

int XpmCreatePixmapFromData(char **lines, Pixmap *pix, Pixmap *clip)
{
	int 		w,h,c;
	rgb_color	col[16];
	uchar		cll[256],*p;

	
	if(lines)
	if(lines[0])
	if(lines[1])
	{
		sscanf(lines[0],"%d %d %d",&w,&h,&c);
	
		cll[lines[1][0]]=0;
		col[0].red=255;
		col[0].green=255;
		col[0].blue=255;
		col[0].alpha=0;

		blist[bcount++]=(*pix)=new BBitmap(BRect(0,0,w-1,h-1),B_COLOR_8_BIT);
		
		p = (uchar *)(*pix)->Bits();
		
		for(int i=1;i<c;i++)
		{
			if(lines[i+1])
			{
				if(strlen(lines[i+1])>15)
				{
					uchar	s;
					int		tmp;
					cll[lines[i+1][0]]=i;
					sscanf(&lines[i+1][5],"%2x",&tmp);
					col[i].red=tmp;
					sscanf(&lines[i+1][9],"%2x",&tmp);
					col[i].green=tmp;
					sscanf(&lines[i+1][13],"%2x",&tmp);
					col[i].blue=tmp;
					col[i].alpha=0;
				}
			}
		}
		c++;
		for(int y=0;y<h;y++)
		{
			for(int x=0;x<w;x++)
			{
				uchar g=cll[lines[y+c][x]];
				
				if(g!=0)
				{
					p[(y*h)+x]=index_for_color(col[g]);
				}
				else
				{
					p[(y*h)+x]=255;	
				}
			}
		}	
	}	
	
	return 0;
}

void draw_pixel(int x, int y, int col)
{
	if((x>0)&&(y>0)&&(x<WINDOW_WIDTH)&&(y<WINDOW_HEIGHT))
	{
		bits[(y*WINDOW_WIDTH)+x]=col;
	}
}

/*
** XPM_MAKE_PIXMAPS
**
*/

void Xpm_make_pixmaps(Sprite *obj)
{
	int i, status;
	Sprite *root_obj=obj;

	bmp->Lock();	
	
	X_color(BLACK);	
	
	for (i=0;i<obj->num_pixmaps;i++) {

		status = XpmCreatePixmapFromData(obj->pixmap_data[i], &obj->pixmaps[i], &obj->clipmasks[i]);


		if(i==0)
		{
			view->FillRect(BRect(0,((vdevice.sizeSy  / 2) + 50),vdevice.sizeSx,vdevice.sizeSy));
			view->DrawBitmapAsync(obj->pixmaps[i],BPoint( ((vdevice.sizeSx - obj->width) / 2), ((vdevice.sizeSy  / 2) + 50)));
			view->Sync();
			X_update(0);
		}
	}

	obj->orig_pixmaps = obj->pixmaps;
	obj->orig_clipmasks = obj->clipmasks;

	while (obj = obj->next) {
		for (i=0;i<obj->num_pixmaps;i++) {
			obj->pixmaps[i] = root_obj->pixmaps[i];
			obj->clipmasks[i] = root_obj->clipmasks[i];
		}

		obj->orig_pixmaps = obj->pixmaps;
		obj->orig_clipmasks = obj->clipmasks;
	};

	X_color(BLACK);

	view->Sync();
	bmp->Unlock();

} /* end XPM_MAKE_PIXMAPS */


/*
** X_ROTATE_PIXMAP_DATA
**
**	Rotates the given pixmap of height h and width w by angle alpha.
**	Alpha ranges from 0 (no rotation) to MAPS_PER_360 (360 degrees).
**
**	Output is not interpolated - "holes" may result.
*/
char **X_rotate_pixmap_data(char **orig_pmap,unsigned int h,unsigned int w,unsigned int alpha)
{
	char **rot_pmap;
	unsigned int i, x, y;
	unsigned int w2, colors, h2, perchar;
	int m, n;
	char pix_val, blank_val;
	float wo2, ho2, x2, y2;

	wo2 = w / 2;
	ho2 = h / 2;

	sscanf(orig_pmap[0], "%d %d %d %d", &w2, &h2, &colors, &perchar);

	rot_pmap = (char **) calloc(colors + h + 1, sizeof(char *));

	for (i=0;i<colors+1;i++) rot_pmap[i] = orig_pmap[i];

	blank_val = orig_pmap[1][0];

	for (i=colors+1;i<colors+1+h;i++) 
		rot_pmap[i] = (char *) calloc(w, sizeof(char));

	for (y=colors+1;y<colors+1+h;y++) 
	{
		for (x=0;x<w;x++) 
		{
			x2 = x - wo2;
			y2 = y - (colors+1) - ho2;

			m = floor(x2 * cos_table[alpha] - y2 * sin_table[alpha] + wo2);
			n = floor(x2 * sin_table[alpha] + y2 * cos_table[alpha] + ho2);
			
			if (m < w && n < h && n > 0) 
			{
				rot_pmap[y][x]=orig_pmap[n+colors+1][m];
			}

		}
	}

	for (m=colors+1;m<h+colors+1;m++) 
	{
		for (n=0;n<w;n++) 
		{
			if (rot_pmap[m][n] == '\0') rot_pmap[m][n] = blank_val;
		}
	}

	return(rot_pmap);
} /* end X_ROTATE_PIXMAP_DATA */




/*
** X_SPIN_PIXMAP360
**
**	Spins a bitmap by 360 degrees, calculating n number of intermediate
**	bitmaps.
**
**	A better way to do this would be to rotate by 90 degrees, then flip
**	horizontally and vertically to obtain the remaining quadrants.
**
**	Inputs: Object with bitmap[0] filled
**	   
*/
void X_spin_pixmap360(Sprite *obj)
{
	unsigned int i, n;

	n = obj->num_pixmaps;

	for (i=1;i<n;i++) {
		obj->pixmap_data[i] = X_rotate_pixmap_data(obj->pixmap_data[0], obj->height, obj->width, i);
	}				
} /* end X_SPIN_PIXMAP360 */

/*
** X_INIT_PIXMAPS
**
*/
void X_init_pixmaps()
{

	int i, j, k, len1, len2;
	
	Xpm_make_pixmaps(P);
	Xpm_make_pixmaps(P->next);
	Xpm_make_pixmaps(Slrb);
	Xpm_make_pixmaps(Homer);
	Xpm_make_pixmaps(Resur);
	Xpm_make_pixmaps(Power_Up);
	Xpm_make_pixmaps(Meta);
	Xpm_make_pixmaps(Flame);
	Xpm_make_pixmaps(Big_O);
	for (i=0;i<4;i++) Xpm_make_pixmaps(Shrd[i]);
	for (i=0;i<MAX_BIG_OIDS;i++) {
		for (j=0;j<OID_DIVIDE;j++) {
			if (i==0 && j==0) {
				Xpm_make_pixmaps(Med_O[0]);
			} else {
				for (k=0;k<Med_O[0]->num_pixmaps;k++) {
					Med_O[i][j].pixmaps[k] = Med_O[0]->pixmaps[k];
					Med_O[i][j].clipmasks[k] = Med_O[0]->clipmasks[k];
				}
			}
		}
	}

	for (i=0;i<MAX_BIG_OIDS * OID_DIVIDE;i++) {
		for (j=0;j<OID_DIVIDE;j++) {
			if (i==0 && j==0) {
				Xpm_make_pixmaps(Sml_O[0]);
			} else {
				for (k=0;k<Sml_O[0]->num_pixmaps;k++) {
					Sml_O[i][j].pixmaps[k] = Sml_O[0]->pixmaps[k];
					Sml_O[i][j].clipmasks[k] = Sml_O[0]->clipmasks[k];
				}
			}
		}
	}
	bmp->Lock();
	
	X_clear();

	view->Sync();
	bmp->Unlock();
} /* end X_INIT_PIXMAPS */


/*
 * X_exit
 *
 *	cleans up before returning the window to normal.
 */
void X_exit()
{
}


/*
** X_clear_key_buffer
**
*/
void X_clear_key_buffer()
{
	globalkey=0;
} /* end X_clear_key_buffer */



/*
** X_get_menu_keys
**
*/
int X_get_menu_keys()
{
	return 0;
} /* end X_GET_MENU_KEYS */



/*
** X_check_keypress
**
*/
void X_check_keypress()
{
/*
	XEvent event;
	KeySym 	key;
	char c[1];
	XComposeStatus compose;

	c[0] = '\0';

	while (XCheckWindowEvent(display, winder, (KeyReleaseMask | KeyPressMask), &event)) {
		XLookupString( (XKeyEvent *) &event, &c[0], 1, &key, &compose);
		switch (event.type) {

		case KeyPress:

			switch (key) {

			case XK_Up:
			case XK_KP_8:
			case XK_R8:
				keyboard_state = keyboard_state | KEY_THRUST1;
				break;
			case XK_Left:
			case XK_KP_4:
			case XK_R10:
				keyboard_state = keyboard_state | KEY_LEFT1;	
				break;
			case XK_Right:
			case XK_KP_6:
			case XK_R12:
				keyboard_state = keyboard_state | KEY_RIGHT1;	
				break;
			case XK_Shift_R:
				keyboard_state = keyboard_state | KEY_FIRE1;
				break;
			case XK_Return:
				keyboard_state = keyboard_state | KEY_HYPER1;
				break;
			case XK_F:
			case XK_f:
				keyboard_state = keyboard_state | KEY_THRUST2;	
				break;
			case XK_C:
			case XK_c:
				keyboard_state = keyboard_state | KEY_LEFT2;	
				break;
			case XK_B:
			case XK_b:
				keyboard_state = keyboard_state | KEY_RIGHT2;	
				break;
			case XK_Shift_L:
				keyboard_state = keyboard_state | KEY_FIRE2;
				break;
			case XK_space:
				keyboard_state = keyboard_state | KEY_HYPER2;
				break;
			case XK_Q:
			case XK_q:
				keyboard_state = keyboard_state | KEY_QUIT;
				break;
			case XK_P:
			case XK_p:
				keyboard_state = keyboard_state | KEY_PAUSE;
				break;
			case XK_Escape:
				keyboard_state = keyboard_state | KEY_ESC;
				break;
			default:
				break;
			}

			break;

		case KeyRelease:

			switch (key) {

			case XK_Up:
			case XK_KP_8:
			case XK_R8:
				keyboard_state = keyboard_state & ~KEY_THRUST1;	
				break;
			case XK_Left:
			case XK_KP_4:
			case XK_R10:
				keyboard_state = keyboard_state & ~KEY_LEFT1;	
				break;
			case XK_Right:
			case XK_KP_6:
			case XK_R12:
				keyboard_state = keyboard_state & ~KEY_RIGHT1;	
				break;
			case XK_Shift_R:
				keyboard_state = keyboard_state & ~KEY_FIRE1;
				break;
			case XK_F:
			case XK_f:
				keyboard_state = keyboard_state & ~KEY_THRUST2;	
				break;
			case XK_C:
			case XK_c:
				keyboard_state = keyboard_state & ~KEY_LEFT2;	
				break;
			case XK_B:
			case XK_b:
				keyboard_state = keyboard_state & ~KEY_RIGHT2;	
				break;
			case XK_Shift_L:
				keyboard_state = keyboard_state & ~KEY_FIRE2;
				break;
			default:
				break;
			}

			break;

		default:

			break;

		}
	}
*/
} /* end X_CHECK_KEYPRESS */


/*
** X_DRAW_OBJECT
**
*/
void X_draw_object(Sprite *obj)
{	
	do 
	{
		if ((obj->state == EXPLODING || obj->state == REVIVING) && obj->death_sprite) X_draw_object(obj->death_sprite);

		if (obj->state <= DYING) 
		{	
			view->DrawBitmapAsync(obj->pixmaps[obj->curr_map],BPoint((int) obj->x - obj->wo2, (int) obj->y - obj->ho2));
		}	
	} while (obj = obj->next_draw);
} /* END X_DRAW_OBJECT */



/*
** X_DRAW_ONE_OBJECT
**
*/
void X_draw_one_object(Sprite *obj,int x,int y)
{
	view->DrawBitmapAsync(obj->pixmaps[0],BPoint(x,y));
} /* END X_DRAW_ONE_OBJECT */




/*
** X_DRAW_LINK
**
** Draws a line from the object to obj->next
*/
void X_draw_link(Sprite *obj,int col)
{
	static int old_x, old_y, old_xl, old_yl;

	if (!(obj->state || obj->next->state)) {

		X_color(BLACK);


		view->MovePenTo(BPoint(obj->ox,obj->oy));
		view->StrokeLine(BPoint(obj->next_draw->ox,obj->next_draw->oy));

		X_color(col);

		view->MovePenTo(BPoint(obj->x,obj->y));
		view->StrokeLine(BPoint(obj->next_draw->x,obj->next_draw->y));

		X_color(BLACK);

		if (old_x) {
			old_x = old_y = 0;
		}

	} else if (!obj->state) {

		X_color(BLACK);

		if (!old_x) {
			old_x = (int) obj->ox;
			old_y = (int) obj->oy;
			old_xl = (int) (obj->ox - 2*link_.length * sin(link_.ang - link_.ang_vec));
			old_yl = (int) (obj->oy - 2*link_.length * cos(link_.ang - link_.ang_vec));
		}

		view->MovePenTo(BPoint(old_x, old_y));
		view->StrokeLine(BPoint(old_xl, old_yl));

		old_x = (int) obj->x;
		old_y = (int) obj->y;
		old_xl = (int) (obj->x - 2*link_.length * sin(link_.ang - link_.ang_vec));
		old_yl = (int) (obj->y - 2*link_.length * cos(link_.ang - link_.ang_vec));

		X_color(col);

		view->MovePenTo(BPoint(old_x, old_y));
		view->StrokeLine(BPoint(old_xl, old_yl));

		X_color(BLACK);

	} else if (!obj->next->state) {

		X_color(BLACK);

		if (!old_x) {
			old_x = (int) obj->next->ox;
			old_y = (int) obj->next->oy;
			old_xl = (int) (obj->next->ox + 2*link_.length * sin(link_.ang - link_.ang_vec));
			old_yl = (int) (obj->next->oy + 2*link_.length * cos(link_.ang - link_.ang_vec));
		}

		view->MovePenTo(BPoint(old_x, old_y));
		view->StrokeLine(BPoint(old_xl, old_yl));

		old_x = (int) obj->next->x;
		old_y = (int) obj->next->y;
		old_xl = (int) (obj->next->x + 2*link_.length * sin(link_.ang - link_.ang_vec));
		old_yl = (int) (obj->next->y + 2*link_.length * cos(link_.ang - link_.ang_vec));

		X_color(col);

		view->MovePenTo(BPoint(old_x, old_y));
		view->StrokeLine(BPoint(old_xl, old_yl));

		X_color(BLACK);
	}
} /* end X_DRAW_LINK */



/*
** X_DRAW_STARS
**
*/
void X_draw_stars(int col,int b)
{
	int x0, y0,  x_rot, y_rot, i,r,rgb;
	static float x_or=0, y_or=0;
	Sprite *player;
	
	player=P;
	
	if(num_players>1)
	{
		x_or +=(-player->x_vec/4-player->next->x_vec/4);
		y_or +=(-player->y_vec/4-player->next->y_vec/4);
	}
	else
	{
		x_or +=(-player->x_vec/2);
		y_or +=(-player->y_vec/2);
	}

	if(y_or<0)y_or=WINDOW_HEIGHT;
	if(x_or<0)x_or=WINDOW_WIDTH;
	if(y_or>WINDOW_HEIGHT)y_or=0;
	if(x_or>WINDOW_WIDTH)x_or=0;

	if (b) num_bursts = 0;

	X_color(col);
	rgb=index_for_color(view->HighColor());

	for (x0=0;x0<vdevice.sizeSx;x0+=WINDOW_WIDTH) 
	{
		for (y0=0;y0<vdevice.sizeSy;y0+=WINDOW_HEIGHT) 
		{
			for (i=0;i<NUM_STARS;i++) 
			{
				x_rot = x0 + ((int)(starfield[i].x+x_or)%WINDOW_WIDTH);
				y_rot = y0 + ((int)(starfield[i].y+y_or)%WINDOW_HEIGHT);
				if(b)
				{
					burst[num_bursts].x = x_rot;
					burst[num_bursts].y = y_rot;
					burst[num_bursts].color = (int) random_num(0.0, 8.0);
					burst[num_bursts].max_radius = (int) random_num(24.0, 48.0);
					burst[num_bursts].radius = 1;
					burst[num_bursts].x_vec = 0;
					burst[num_bursts].y_vec = 0;
					burst[num_bursts++].dir = 1;
				} 
				else 
				{
					draw_pixel(x_rot,y_rot,rgb);
				}
			}

		}
	}

	X_color(BLACK);
} /* end X_XDRAW_STARS */



 
/*
** X_DRAW_STATUS_BAR
**
*/
void X_draw_status_bar()
{	
	char str1[30], str2[30];
	int i;
	int new_val;

	sprintf(str1, "%s Score: %d", P->name, P->score);
	if (num_players > 1) sprintf(str2, "%s Score: %d", P->next->name, P->next->score);

	X_color(BLACK);
	view->FillRect(BRect(0, 0, vdevice.sizeSx, P->height + 3));
	if (num_players > 1) 
	{
		X_draw_one_object(P->next, 5, 0);
		X_draw_one_object(P,WINDOW_WIDTH-P->width-5, 0);

		new_val = (int) (wo7 * P->next->engine / 1000);
		if (P->next->engine > 750) 
		{
			X_color(RED);
		} 
		else if (P->next->engine > 500) 
		{
			X_color(YELLOW);
		} 
		else X_color(GREEN);
		view->FillRect(BRect((WINDOW_WIDTH/2)-new_val-4, P->height/5, (WINDOW_WIDTH/2)-4, P->height-(P->height/5)));

		new_val = (int) (wo7 * P->engine / 1000);
		if (P->engine > 750) 
		{
			X_color(RED);
		} 
		else if (P->engine > 500) 
		{
			X_color(YELLOW);
		} 
		else X_color(GREEN);
		view->FillRect(BRect((WINDOW_WIDTH/2)+4, P->height/5, (WINDOW_WIDTH/2)+4+new_val, P->height-(P->height/5)));
	} 
	else 
	{
		for (i=0;i<P->lives;i++) 
		{
			X_draw_one_object(P, i * (int) (P->width + 5) + 5, 0);
		}			
		new_val = (int) (wo7 * P->engine / 1000);
		if (P->engine > 750) 
		{
			X_color(RED);
		} 
		else if (P->engine > 500) 
		{
			X_color(YELLOW);
		} 
		else X_color(GREEN);
		view->FillRect(BRect(WINDOW_WIDTH-wo7, P->height/5, WINDOW_WIDTH-wo7+new_val, P->height-(P->height/5)));
	}

	X_color(WHITE);

	if (num_players > 1) 
	{
		view->DrawString(str2,BPoint(10 + P->width, P->height - 7));		
		view->DrawString(str1,BPoint(WINDOW_WIDTH-P->width-10-view->StringWidth(str1), P->height - 7));
	}
	else 
	{
		view->DrawString(str1,BPoint(10 + (P->lives > 0 ? P->lives : 1) * (P->width + 5), P->height - 7));
	}

	view->MovePenTo(BPoint(0, P->height + 3));
	view->StrokeLine(BPoint(vdevice.sizeSx, P->height + 3));

	X_color(BLACK);

} /* end X_DRAW_STATUS_BAR */



/*
** X_update_status_bar
**
*/
void X_update_status_bar()
{
} /* end UPDATE_STATUS_BAR */



/*
 * X_FLASH_SCREEN
 *
 * Clear the screen (or current buffer )
 */
void X_flash_screen(int d)
{
	memset(bits,255,WINDOW_WIDTH*WINDOW_HEIGHT);
	X_update(0);
} /* end X_FLASH_SCREEN */



/*
** X_DRAW_THRUST
**
*/
void X_draw_thrust(Sprite *obj)
{
	int i, x1, y1, x2, y2, m1, n1, m2, n2;

	x1 = thrust_pts_x[0];
	y1 = thrust_pts_y[0];
	m1 = x1*cos_table[obj->curr_map] - y1*sin_table[obj->curr_map];
	n1 = -x1*sin_table[obj->curr_map] - y1*cos_table[obj->curr_map];

	X_color(RED);

	for (i=1;i<N_THRUST_PTS;i++) {
		x2 = thrust_pts_x[i];
		y2 = thrust_pts_y[i];
		m2 = x2*cos_table[obj->curr_map] - y2*sin_table[obj->curr_map];
		n2 = -x2*sin_table[obj->curr_map] - y2*cos_table[obj->curr_map];
	
		view->MovePenTo(BPoint( m1 + (int) obj->x, n1 + (int) obj->y));
		view->StrokeLine(BPoint(m2 + (int) obj->x, n2 + (int) obj->y));

		m1 = m2;
		n1 = n2;
	}

	X_color(BLACK);
	
} /* end X_DRAW_THRUST */



/*
** X_DRAW_BURSTS
**
*/
void X_draw_bursts(int flag)
{
	int i,j,x,y;
	

	for (i=0;i<num_bursts;i++) {

		if (flag) {
			X_color(burst[i].color);
		} else {
			X_color(BLACK);
		}
		
		int rgb=index_for_color(view->HighColor());

		for (j=0;j<16;j++) {

			x = burst[i].x + (int) burst[i].radius * cos_table[j << 1];
			y = burst[i].y + (int) burst[i].radius * sin_table[j << 1];

			draw_pixel(x,y,rgb);
		}

	}

	X_color(BLACK);
} /* end X_DRAW_BURSTS */



/*
** X_DRAW_SHOTS
**
*/
void X_draw_shots(Sprite *obj)
{
	int i, j;
	float s, c;
	struct Shot *curr;

	do {

		if (obj->shots) {

			X_color(obj->shot_color);

			for (i=0;i<obj->shots;i++) {

				curr = obj->S[i];

				for (j=0;j<4;j++)
				{
					view->MovePenTo(BPoint(curr->x + curr->pts_x[j],curr->y + curr->pts_y[j]));
					view->StrokeLine(BPoint(curr->x + curr->pts_x[j+1], curr->y + curr->pts_y[j+1]));
				}
			}
		}

	} while (obj = obj->next);

	X_color(BLACK);
} /* end X_DRAW_SHOTS */


/*
** X_CLEAR_SHOTS
**
*/
void X_clear_shots(Sprite *obj)
{
} /* end X_CLEAR_SHOTS */


/*
** X_CLEAR_OBJECT
**
*/
void X_clear_object(Sprite *obj)
{
} /* end X_CLEAR_OBJECT */


/*
** X_CLEAR_ONE_OBJECT
**
*/
void X_clear_one_object(Sprite *obj)
{
} /* end X_CLEAR_ONE_OBJECT */



/*
**
** X_COPY_STATUS_BAR_TO_WINDOW
**
*/
void X_copy_status_bar_to_window()
{
} /* end X_COPY_STATUS_BAR_TO_WINDOW */




/*
 * X_COPY_OBJECT_TO_WINDOW
 *
 *	Swap the back and from buffers. (Really, just copy the
 *	back buffer to the screen).
 */
void X_copy_object_to_window(Sprite *obj)
{
} /* end X_COPY_OBJECT_TO_WINDOW */



/*
 * X_draw
 *
 *	draws a line from the current graphics position to (x, y).
 *
 * Note: (0, 0) is defined as the top left of the window in X (easy
 * to forget).
 */
void X_draw(int x, int y)
{
}

/*
** X_point
**
**	Draw a point!
**
*/
void X_point(int x, int y)
{
	int rgb=index_for_color(view->HighColor());
	draw_pixel(x,y,rgb);
} /* end X_point */


/*
 * X_getkey
 *
 *	grab a character from the keyboard - blocks until one is there.
 */
int X_getkey()
{
	ulong key;
	globalkey=0;
	while(!(key=globalkey))snooze(20000);
	return(key);	
}

/*
** X_UPDATE
**
** Flush the X server's queue and handle visilibity and
** window resizing events.
**
*/
void X_update(unsigned int game_flag)
{
	if(window_view)window_view->RefreshAll();
} /* end X_UPDATE */


/*
 * X_checkkey
 *
 *	Check if there has been a keyboard key pressed.
 *	and return it if there is.
 */
int X_checkkey()
{
	return('1');
}

/*
 * X_locator
 *
 *	return the window location of the cursor, plus which mouse button,
 * if any, is been pressed.
 */
int X_locator(int *wx, int *wy)
{
	return 0;
}

/*
 * X_clear
 *
 * Clear the screen (or current buffer )to current colour
 */
void X_clear()
{
	X_color(BLACK);
	view->FillRect(BRect(0,0,WINDOW_WIDTH,WINDOW_HEIGHT));
}

/*
 * X_color
 *
 *	set the current drawing color index.
 */
void X_color(int ind)
{
	switch(ind)
	{
		default:
		case	BLACK:
				view->SetHighColor(0,0,0);
				break;
		case	RED:
				view->SetHighColor(255,0,0);
				break;
		case	GREEN:
				view->SetHighColor(0,255,0);
				break;
		case	YELLOW:
				view->SetHighColor(255,255,0);
				break;
		case	BLUE:
				view->SetHighColor(0,0,255);
				break;
		case	MAGENTA:
				view->SetHighColor(255,0,255);
				break;
		case	CYAN:
				view->SetHighColor(0,255,255);
				break;
		case	WHITE:
				view->SetHighColor(255,255,255);
				break;
	}
}

/*
 * X_mapcolor
 *
 *	change index i in the color map to the appropriate r, g, b, value.
 */
int X_mapcolor(int i,int r,int g,int b)
{
	return(0);
}
	
/*
 * X_font
 *
 *   Set up a hardware font. Return 1 on success 0 otherwise.
 *
 */
X_font(char	*fontfile)
{
	return(1);
}

/* 
 * X_char
 *
 *	 outputs one char - is more complicated for other devices
 */
void X_char(char	c)
{
	view->DrawChar(c,BPoint(vdevice.cpVx, (int)(vdevice.sizeSy - vdevice.cpVy)));
}

/*
 * X_string
 *
 *	Display a string centered at the specified height
 */
void X_string(char *s, int y)
{
	float x;
	x=view->StringWidth(s);
	view->DrawString(s,BPoint((vdevice.sizeSx / 2 - x / 2), y));
}

/*
 * X_fill
 *
 *	fill a polygon
 */
void X_fill(int	n, int x[], int y[])
{
}

#define	GC_COPY_MASK	~0

/*
 * X_backbuf
 *
 *	Set up double buffering by allocating the back buffer and
 *	setting drawing into it.
 */
void X_backbuf()
{
}

/*
 * X_swapbuf
 *
 *	Swap the back and from buffers. (Really, just copy the
 *	back buffer to the screen).
 */
void X_swapbuf()
{
}

/*
 * X_frontbuf
 *
 *	Make sure we draw to the screen.
 */
void X_frontbuf()
{
}

/*
** X_PRINT_QUEUE
**
*/
void X_print_queue()
{
} /* end X_PRINT_QUEUE */
