/*
 *  Oids.cpp - 	 Oids effect
 *               Written by Tinic Urou
 *               Public domain
 */

#include <AppKit.h>
#include <InterfaceKit.h>
#include <KernelKit.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>

#include "oids.h"

// Constants

const 	ULONG 			APP_SIGNATURE = '????';

#define MSG_REFRESH	1

// Globals

		BBitmap			*bmp;
		BView			*view;
		OidsView		*window_view;
		OidsWindow		*window;
		uchar			colors[256],*bits,*buf;	
		ulong			globalkey;	
		sem_id			upd_sem;
		
// Window/Application messages

const 	LONG			MSG_REDRAW = 1;

// Create application object and start it
short main(short argc, char **argv)
{	
	Oids *the_app;

	srand(system_time());

	the_app = new Oids();
	the_app->Run();
	delete the_app;

	return 0;
}

/*
 *  Application constructor
 */

Oids::Oids() : BApplication(APP_SIGNATURE)
{
	the_window = NULL;
}

/*
 *	Thats from me!
 */

void Oids::AboutRequested(void) 
{
	char str[256];
	sprintf(str,"XOIDS for BeOS by Tinic Urou\n<5uro@informatik.uni-hamburg.de>\nFreely distributable.");
	BAlert *the_alert = new BAlert("", str, "OK");
	the_alert->Go();
}

void Oids::init_game()
{
	int i;
	Sprite *the_obj;

	acquire_sem(upd_sem);

	float wid1=0, wid2=0;
	char *str1 = "ALLOCATING MEMORY AND ROTATING BITMAPS";
	char *str2 = "Be patient, or else you won't get to play";

	bmp->Lock();

	X_color(BLACK);

	X_clear();

	view->SetFontName("Arial MT Bold");
	view->SetFontSize(15);
	view->SetDrawingMode(B_OP_OVER);

	wid1 = view->StringWidth(str1);
	wid2 = view->StringWidth(str2);

	X_color(YELLOW);
	view->DrawString(str1,BPoint((WINDOW_WIDTH / 2) - wid1 / 2, WINDOW_HEIGHT / 2 - 50));
	X_color(RED);
	view->DrawString(str2,BPoint((WINDOW_WIDTH / 2) - wid2 / 2, WINDOW_HEIGHT / 2));
	X_color(BLACK);	
	
	view->Sync();
	bmp->Unlock();
	
	if(window_view)window_view->RefreshAll();

	 
	/* Build the trig tables */

	for (i=0;i<MAPS_PER_360;i++) 
	{
		cos_table[i] = cos(i*(2*PI)/MAPS_PER_360);
		sin_table[i] = sin(i*(2*PI)/MAPS_PER_360);
	}

	/* Reset all our globals */

	keyboard_state = 0;
	timer = TIMER_VAL;
	num_bursts = 0;
	extra_man_scr = 0;
	oids_shot = 0;
	game_over = 0;
	goal_score = 1;
	warp_levels = 0;
	leave = 0;
	level = 1;

	/* Set the window size to the default values */

	vdevice.sizeSx = WINDOW_WIDTH;
	vdevice.sizeSy = WINDOW_HEIGHT;

	bmp->Lock();

	init_all_objects(0);

	X_init();

	X_init_pixmaps();

	X_clear_key_buffer();
	view->Sync();
	bmp->Unlock();

	main_menu();

	bmp->Lock();
	X_backbuf();
	X_clear();
	update_score();

	view->Sync();
	bmp->Unlock();

	the_obj = P;

	do 
	{
		revive_player(the_obj);
	} 
	while (the_obj = the_obj->next);
	
	release_sem(upd_sem);
	
	window->PostMessage(MSG_REFRESH);
}

/*
 *  Open window and start rendering
 */

void Oids::ReadyToRun(void)
{
	if((upd_sem=create_sem(1,"XOIDS updater"))<B_NO_ERROR) exit(-1);

	// Open window
		
	window = the_window = new OidsWindow(WINDOW_WIDTH,WINDOW_HEIGHT);
	the_window->Show();
	
	bmp=new BBitmap(BRect(0, 0, WINDOW_WIDTH-1, WINDOW_HEIGHT-1),B_COLOR_8_BIT,TRUE);	
	view=new BView(BRect(0, 0, WINDOW_WIDTH-1, WINDOW_HEIGHT-1),"Test",NULL,NULL);
	bmp->Lock();
	bmp->AddChild(view);
	view->SetDrawingMode(B_OP_OVER);
	bmp->Unlock();
	buf=bits=(uchar *)bmp->Bits();

	// We dont want to block the application, so...	
	resume_thread(spawn_thread((thread_entry)init_game,"init_game",B_NORMAL_PRIORITY,NULL));
}

/*
 *  Quit requested (either by menu or by closing the window)
 */

bool Oids::QuitRequested(void)
{
	// Make sure that the window closes first
	
	acquire_sem(upd_sem);
	
	if (BApplication::QuitRequested()) 
	{
		free_all_objects();
		if(bmp) delete bmp;
		delete_sem(upd_sem);
		return TRUE;
	}
	return FALSE;
}

/*
 *  Window constructor
 */

OidsWindow::OidsWindow(short width, short height) : BWindow(BRect(0, 0, width-1, height-1), "XOIDS by Tinic", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{		
	// Move window to right position
	MoveTo(0,0);
	
	// Create bitmap view
	Lock();
	window_view = main_view = new OidsView(Bounds());
	AddChild(window_view);
	window_view->MakeFocus();
	Unlock();
	
	Lock();
	ResizeTo(WINDOW_WIDTH-1, WINDOW_HEIGHT-1);
	main_view->ResizeTo(WINDOW_WIDTH-1, WINDOW_HEIGHT-1);
	Unlock();
}


/*
 *  Closing the window quits the program
 */

bool OidsWindow::QuitRequested(void)
{	
	window_view=0;
	window=0;
	be_app->PostMessage(B_QUIT_REQUESTED);
	return TRUE;
}


/*
 *  Handles redraw messages
 */

void OidsWindow::MessageReceived(BMessage *msg)
{
	switch (msg->what) 
	{
		case	MSG_REFRESH:
				// Do not wait for Refresh! This could lend to problems...
				resume_thread(spawn_thread((thread_entry)Refresh,"XOIDS Thread",B_LOW_PRIORITY,this));
				break;
		default:
				BWindow::MessageReceived(msg);
				break;
	}
}


OidsView::OidsView(BRect frame):BView(frame,"OidsView",NULL,B_WILL_DRAW | B_PULSE_NEEDED)
{
	MakeFocus();
}

void OidsView::Draw(BRect update)
{
	DrawBitmap(bmp,BPoint(0,0));
}

void OidsView::RefreshAll()
{
	Window()->Lock();
	Draw(BRect(0,0,0,0));
	Window()->Unlock();
}

void OidsWindow::Refresh(OidsWindow *w)
{
	int 		i;
	Sprite 		*the_obj;
	
	if(acquire_sem(upd_sem)==B_NO_ERROR)
	{
		if(game_over < 100 && !leave) 
		{
			key_info ki;

			/* Draw stuff */
			
			bmp->Lock();
		
			/* handle player(s) first */
		
			X_clear();

			X_draw_stars(WHITE, 0);
		
			the_obj = P;

			do 
			{
				X_draw_object(the_obj);
						
				if (the_obj->thrust) X_draw_thrust(the_obj);
				switch(the_obj->state) 
				{
					case 	WARPING:
							if (the_obj->state_ctr == HYPER_DELAY) warp_object(the_obj);
							break;
					case 	DEAD:
							if (!game_over) revive_player(the_obj);
							break;
				}

			} while (the_obj = the_obj->next);

			/* Handle end of one player game */

			if (num_players == 1 && P->lives == 0) 
			{
				game_over = 1;
				P->lives = -1;
			}

			/* Check for bonus life in the one player game */

			if (num_players == 1 && (P->score - extra_man_scr) >= 10000) 
			{
				if (++P->lives > 20) P->lives = 20;
				extra_man_scr += 10000;
				update_score();
				make_burst(P, -1);
			}

			/* Handle end of two player game */

			if (num_players > 1 && !game_over && (P->score >= goal_array[goal_score] || P->next->score >= goal_array[goal_score])) {
				game_over = 1;
			}

			/* Drawing to offscreen pixmap - could clean all this */
			/* stuff up with some a list of all the game objects */
			/* (C++ would come in handy here) */

			X_draw_object(Big_O);
			X_draw_shots(P);
			X_draw_object(Slrb);
			X_draw_shots(Slrb);
			X_draw_object(Homer);
			X_draw_object(Resur);
			if (!Resur->state && Resur->rotation) 
			{
				Flame->curr_map = ((int) random_num(0.0, 6.0)) % 2;
				Flame->x = Resur->x;
				Flame->y = Resur->y;
				X_draw_object(Flame);
			}
			X_draw_object(Power_Up);
			X_draw_object(Meta);

			/* Copy from offscreen pixmap to window */

			/* Drawing to the window */
			/* Update bursts */	

			X_draw_bursts(0);
			update_bursts();
			X_draw_bursts(1);

			if (coop) X_draw_link(P, GREEN);

			X_draw_status_bar();  /* MUST follow all drawing */
			
			/* Get stuff from keyboard, mouse, etc... */

			if (!game_over)
			{
				if(window)
				{
					window->Lock();
					window_view->GetKeys(&ki,FALSE);
					window->Unlock();
				}

				if ( (ki.key_states[0x57>>3] & (1<<(7-(0x57%8)))))
				{
					keyboard_state = keyboard_state | KEY_THRUST1;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_THRUST1;
				}
			
				if ( (ki.key_states[0x61>>3] & (1<<(7-(0x61%8)))) ) 
				{				
					keyboard_state = keyboard_state | KEY_LEFT1;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_LEFT1;
				}

				if ( (ki.key_states[0x63>>3] & (1<<(7-(0x63%8)))) ) 
				{
					keyboard_state = keyboard_state | KEY_RIGHT1;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_RIGHT1;
				}
		
				if ( (ki.key_states[0x5f>>3] & (1<<(7-(0x5f%8))))) 
				{
					keyboard_state = keyboard_state | KEY_FIRE1;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_FIRE1;
				}

				// ENTER
				if ( (ki.key_states[0x47>>3] & (1<<(7-(0x47%8))))) 
				{
					keyboard_state = keyboard_state | KEY_HYPER1;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_HYPER1;
				}

				if ( (ki.key_states[0x27>>3] & (1<<(7-(0x27%8))))) 
				{
					keyboard_state = keyboard_state | KEY_QUIT;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_QUIT;
				}
				
				// P
				if ( (ki.key_states[0x30>>3] & (1<<(7-(0x30%8))))) 
				{
					keyboard_state = keyboard_state | KEY_PAUSE;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_PAUSE;
				}

				if ( (ki.key_states[0x01>>3] & (1<<(7-(0x01%8))))) 
				{
					keyboard_state = keyboard_state | KEY_ESC;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_ESC;
				}

				// D
				if ( (ki.key_states[0x3e>>3] & (1<<(7-(0x3e%8))))) 
				{
					keyboard_state = keyboard_state | KEY_THRUST2;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_THRUST2;
				}
				
				// X
				if ( (ki.key_states[0x4d>>3] & (1<<(7-(0x4d%8))))) 
				{
					keyboard_state = keyboard_state | KEY_LEFT2;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_LEFT2;
				}
				
				// V
				if ( (ki.key_states[0x4f>>3] & (1<<(7-(0x4f%8))))) 
				{
					keyboard_state = keyboard_state | KEY_RIGHT2;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_RIGHT2;
				}
				
				// Left ALT
				if ( (ki.key_states[0x5d>>3] & (1<<(7-(0x5d%8))))) 
				{
					keyboard_state = keyboard_state | KEY_FIRE2;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_FIRE2;
				}
				
				// SPACE
				if ( (ki.key_states[0x5e>>3] & (1<<(7-(0x5e%8))))) 
				{
					keyboard_state = keyboard_state | KEY_HYPER2;
				}	
				else
				{
					keyboard_state = keyboard_state & ~KEY_HYPER2;
				}
				
				check_input();
			}

			/* Update sprite positions */

			update_object(P);
			if (coop) link_players();
			update_object(Slrb);
			update_object(Homer);
			update_object(Resur);
			update_object(Big_O);
			update_object(Power_Up);
			update_object(Meta);

			/* Update shot positions */

			if (the_obj = update_shots(P)) {
				if (!(coop && (the_obj == P || the_obj == P->next)))
					(*the_obj->kill_func)(the_obj);
			}

			if (the_obj = update_shots(Slrb)) {
				(*the_obj->kill_func)(the_obj);
			}

			if (shot_clock) shot_clock--;

			/* Check for collisions (ooh, aahh) */

			/* Player collisions */

			check_collision(P, Big_O, 1, 1);

			if (num_players > 1) simple_collision(P, P->next, 1, 1);

			/* Alien collisions */

			if (Slrb->state <= DYING) {
				check_collision(Slrb, Big_O, 1, 1);
				simple_collision(P, Slrb, 1, 1);
				if (num_players > 1) simple_collision(P->next, Slrb, 1, 1);
				if (Slrb->wpn != GUN && Slrb->shots < Weapon[Slrb->wpn].max_shots && !shot_clock) create_new_shot(Slrb);	

			}

			if (!Homer->state) {
				alien_lock_on(Homer);
				draw_homer_trail(0);
				check_collision(Homer, Big_O, 1, 1);
				simple_collision(P, Homer, 1, 1);
				if (!Slrb->state) simple_collision(Homer, Slrb, 1, 1);
				if (num_players > 1) simple_collision(P->next, Homer, 1, 1);
			}

			if (!Resur->state) {
				check_collision(Resur, Big_O, 1, 0);
				simple_collision(P, Resur, 1, 1);
				if (!Slrb->state) simple_collision(Resur, Slrb, 1, 1);
				if (!Homer->state) simple_collision(Resur, Homer, 1, 1);
				if (num_players > 1) simple_collision(P->next, Resur, 1, 1);
			}

			if (!Power_Up->state) {
				the_obj = NULL;
				check_collision(Power_Up, Big_O, 1, 1);
				if (simple_collision(P, Power_Up, 0, 0)) the_obj = P;
				if (simple_collision(Slrb, Power_Up, 0, 0)) the_obj = Slrb;
				if (num_players > 1 && simple_collision(P->next, Power_Up, 0, 0)) the_obj = P->next;

				if (the_obj) kill_power_up(Power_Up, the_obj);
			}

			if (!Meta->state) {
				the_obj = NULL;
				check_collision(Meta, Big_O, 1, 1);
				if (simple_collision(P, Meta, 0, 0)) the_obj = P;
				if (simple_collision(Slrb, Meta, 0, 0)) the_obj = Slrb;
				if (num_players > 1 && simple_collision(P->next, Meta, 0, 0)) the_obj = P->next;

				if (the_obj) kill_power_up(Meta, the_obj);
			}	

			/* Update timer (used by oid rotation) */	

			if (timer) timer--;
			if (!timer) {
				test_for_aliens();
				timer = TIMER_VAL;
			}

			/* Finished the level, so warp outta here */

			if (warp_levels) {
				if (warp_levels == MAPS_PER_360) X_draw_stars(WHITE, 1);
				warp_levels--;
				draw_level_warp();
				if (!warp_levels) {
					num_bursts = 0;
					X_flash_screen(2);
				}
			}

			// Sync all Sprites
			
			view->Sync();
			bmp->Unlock();
			X_update(0);

			/* If game is over, wait for the end */
			
			if (game_over) 
			{
				game_over++;
			
				draw_game_over();
				X_getkey();		
						
				leave=1;
			}
		}
		else 
		{				
			main_menu();

			bmp->Lock();		
			X_backbuf();
			X_clear();
			update_score();

			view->Sync();
			bmp->Unlock();

			the_obj = P;

			leave=0;
			game_over=0;
			init_all_objects(1);

			/* Bring the players to life */

			do 
			{
				revive_player(the_obj);
			} 
			while (the_obj = the_obj->next);
		
		}
		w->PostMessage(MSG_REFRESH);
		release_sem(upd_sem);
	}
}

void OidsView::KeyDown(ulong aKey)
{
	// Quick getkey(); hack...
	
	globalkey=aKey;
}
